@@ -35,11 +35,15 @@
 #include <string.h>
 
 #include <libgnome/gnome-i18n.h>
+#include <gdk/gdkkeysyms.h>
+
 #include "gedit-view.h"
 #include "gedit-debug.h"
 #include "gedit-menus.h"
 #include "gedit-prefs-manager.h"
 
+#include "gedit-marshal.h"
+
 #define MIN_NUMBER_WINDOW_WIDTH 20
 
 struct _GeditViewPrivate
@@ -68,6 +72,14 @@ static GtkTargetEntry drop_types[] = {
 
 static gint n_drop_types = sizeof (drop_types) / sizeof (drop_types[0]);
 
+enum
+{
+ 	POPULATE_POPUP,
+	LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL] = { 0 };
+
 static void gedit_view_class_init 	(GeditViewClass	*klass);
 static void gedit_view_init 		(GeditView 	*view);
 static void gedit_view_finalize 	(GObject 	*object);
@@ -97,6 +109,7 @@ static void gedit_view_dnd_drop (GtkTextView *view,
 			     guint info,
 			     guint time,
 			     gpointer data);
+static gint gedit_view_key_press_cb (GtkWidget *widget, GdkEventKey *event, GeditView *view);
 
 static GtkVBoxClass *parent_class = NULL;
 
@@ -165,7 +178,17 @@ gedit_view_class_init (GeditViewClass *klass)
   	object_class->finalize = gedit_view_finalize;
 
 	GTK_WIDGET_CLASS (klass)->grab_focus = gedit_view_grab_focus;
-	
+
+	signals[POPULATE_POPUP] =
+		g_signal_new ("populate_popup",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GeditViewClass, populate_popup),
+			      NULL, NULL,
+			      gedit_marshal_VOID__OBJECT,
+			      G_TYPE_NONE,
+			      1,
+			      GTK_TYPE_MENU);
 }
 
 /* This function is taken from gtk+/tests/testtext.c */
@@ -363,6 +386,33 @@ gedit_view_line_numbers_expose (GtkWidget      *widget,
 	return FALSE;
 }
 
+/* when the user right-clicks on a word, we move the cursor to the 
+ * location of the clicked-upon word.
+ */
+static gboolean
+button_press_event (GtkTextView *view, GdkEventButton *event, gpointer data) 
+{
+	if (event->button == 3) 
+	{
+		gint x, y;
+		GtkTextIter iter;
+		GtkTextIter start, end;
+
+		gtk_text_view_window_to_buffer_coords (view, 
+				GTK_TEXT_WINDOW_TEXT, 
+				event->x, event->y,
+				&x, &y);
+		
+		gtk_text_view_get_iter_at_location(view, &iter, x, y);
+
+		if (!(gtk_text_buffer_get_selection_bounds (
+					gtk_text_view_get_buffer (view), &start, &end) &&          
+		    gtk_text_iter_in_range (&iter, &start, &end)))
+			gtk_text_buffer_place_cursor (gtk_text_view_get_buffer (view), &iter);
+	}
+	return FALSE; /* false: let gtk process this event, too.
+			 we don't want to eat any events. */
+}
 
 static void 
 gedit_view_init (GeditView  *view)
@@ -427,9 +477,14 @@ gedit_view_init (GeditView  *view)
 	gtk_text_view_set_right_margin (GTK_TEXT_VIEW (view->priv->text_view), 2);
 
 	GTK_WIDGET_SET_FLAGS (GTK_WIDGET (view), GTK_CAN_FOCUS);
+	GTK_WIDGET_SET_FLAGS (GTK_WIDGET (view->priv->text_view), GTK_CAN_FOCUS);
 
 	g_signal_connect (G_OBJECT (view->priv->text_view), "populate-popup",
 			  G_CALLBACK (gedit_view_populate_popup), view);
+
+	g_signal_connect (G_OBJECT (view->priv->text_view), "button-press-event",
+			  G_CALLBACK (button_press_event), NULL);
+
 }
 
 static void 
@@ -532,6 +587,12 @@ gedit_view_new (GeditDocument *doc)
 			  G_CALLBACK (gedit_view_doc_readonly_changed_handler),
 			  view);
 
+	g_signal_connect (G_OBJECT (view->priv->text_view),
+			  "key_press_event",
+			  G_CALLBACK (gedit_view_key_press_cb),
+			  view);
+
+
 	gtk_text_view_set_editable (view->priv->text_view, !gedit_document_is_readonly (doc));	
 
 	
@@ -1019,7 +1080,9 @@ gedit_view_populate_popup (GtkTextView *textview, GtkMenu *menu, gpointer data)
 	g_signal_connect (G_OBJECT (menu_item), "activate",
 		      	  G_CALLBACK (gedit_view_undo_activate_callback), doc);
 	gtk_widget_show (menu_item);
-	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);	
+	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
+
+	g_signal_emit (G_OBJECT (view), signals[POPULATE_POPUP], 0, menu);
 }
 
 static void 
@@ -1074,3 +1137,124 @@ gedit_view_dnd_drop (GtkTextView *view,
 	}
 }
 
+GtkTextView *
+gedit_view_get_gtk_text_view (const GeditView *view)
+{
+	gedit_debug (DEBUG_VIEW, "");
+
+	g_return_val_if_fail (GEDIT_IS_VIEW (view), NULL);
+
+	return view->priv->text_view;
+}
+
+static gchar*
+compute_indentation (GeditView *view, gint line)
+{
+	GtkTextIter start;
+	GtkTextIter end;
+	gunichar ch;
+
+	gedit_debug (DEBUG_VIEW, "");
+
+	gtk_text_buffer_get_iter_at_line (GTK_TEXT_BUFFER (view->priv->document), 
+			&start, line);
+
+	end = start;
+
+	ch = gtk_text_iter_get_char (&end);
+	while (g_unichar_isspace (ch) && ch != '\n')
+	{
+		if (!gtk_text_iter_forward_char (&end))
+			break;
+
+		ch = gtk_text_iter_get_char (&end);
+	}
+
+	if (gtk_text_iter_equal (&start, &end))
+		return NULL;
+
+	return gtk_text_iter_get_slice (&start, &end);
+}
+
+static gint
+gedit_view_key_press_cb (GtkWidget *widget, GdkEventKey *event, GeditView *view)
+{
+	GtkTextBuffer *buf;
+	GtkTextIter cur;
+	GtkTextMark *mark;
+	gint key;
+	gboolean auto_indent;
+	gboolean insert_spaces;
+
+	auto_indent = gedit_prefs_manager_get_auto_indent();
+	insert_spaces = gedit_prefs_manager_get_insert_spaces ();
+
+	buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));
+
+	key = event->keyval;
+
+	mark = gtk_text_buffer_get_mark (buf, "insert");
+	gtk_text_buffer_get_iter_at_mark (buf, &cur, mark);
+	
+	if ((key == GDK_Return) && auto_indent && gtk_text_iter_ends_line (&cur)) 
+	{
+		/* Auto-indent means that when you press ENTER at the end of a
+		 * line, the new line is automatically indented at the same
+		 * level as the previous line.
+		 */
+		gchar *indent = NULL;
+
+		/* Calculate line indentation and create indent string. */
+		indent = compute_indentation (view, gtk_text_iter_get_line (&cur));
+
+		if (indent != NULL)
+		{
+			/* Insert new line and auto-indent. */
+			gtk_text_buffer_begin_user_action (buf);
+			gtk_text_buffer_insert (buf, &cur, "\n", 1);
+			gtk_text_buffer_insert (buf, &cur, indent, strlen (indent));
+			g_free (indent);
+			gtk_text_buffer_end_user_action (buf);
+			gtk_text_view_scroll_mark_onscreen (GTK_TEXT_VIEW (widget),
+							    gtk_text_buffer_get_insert (buf));
+			return TRUE;
+		}
+	}
+
+	if ((key == GDK_Tab) && insert_spaces)
+	{
+		gint cur_pos;
+		gint num_of_equivalent_spaces;
+		gint tabs_size;
+		gint i;
+		gchar *spaces;
+
+		tabs_size = gedit_prefs_manager_get_tabs_size (); 
+		
+		cur_pos = gtk_text_iter_get_line_offset (&cur);
+
+		num_of_equivalent_spaces = tabs_size - (cur_pos % tabs_size); 
+
+		spaces = g_malloc (num_of_equivalent_spaces + 1);
+		
+		for (i = 0; i < num_of_equivalent_spaces; i++)
+			spaces [i] = ' ';
+		
+		spaces [num_of_equivalent_spaces] = '\0';
+		
+		gtk_text_buffer_begin_user_action (buf);
+		gtk_text_buffer_insert (buf,  &cur, spaces, num_of_equivalent_spaces);
+		gtk_text_buffer_end_user_action (buf);
+
+		gtk_text_view_scroll_mark_onscreen (GTK_TEXT_VIEW (widget),
+						    gtk_text_buffer_get_insert (buf));
+		
+		g_free (spaces);
+
+		return TRUE;
+	}
+		
+	return GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
+}
+
+
